import urllib
import gzip
import json
import re
import os
import datetime

orangetvurl = 'http://chaines-tv.orange.fr/'

def jsonLoad(filename) :
  try :
    with open(filename, 'r') as f :
      return json.load(f)
  except :
    return None

def jsonGzipLoad(self, filename) :
  try :
    return json.load(open(filename, 'r'))
  except :
    return None

class Channels :
  
  channels = dict()
  cachepath = ''
  iconpath = ''
  licences = []
  
  def __init__(self, cachepath) :
    self.cachepath = cachepath
    
  def fetchChannels(self) : # can raise urllib.error (inherits from IOError)
    url = orangetvurl + 'pfs-webapp/live/servicePlan.json'
    urllib.urlretrieve(url, os.path.join(self.cachepath, 'servicePlan.json'))
    info = jsonLoad(os.path.join(self.cachepath, 'servicePlan.json'))
    if info != None and info.has_key('response') :
      for channel in info['response'] :
        programs = list()
        for program in channel['programs'] :
          programs.append(Program(program['title'], program['genre'], datetime.datetime.fromtimestamp(program['diffusionDate']/1000), program['duration']*60))
        self.channels[channel['channelOrder']] = Channel(channel['channelId'], channel['name'], channel['logo'], programs, channel['licenceType'])
  
  def getChannels(self) :  # can raise urllib.error (inherits from IOError)
    if len(self.channels) == 0 :
      self.fetchChannels()
    return self.channels
  
  def getChannelById(self, channelId) : 
    """Get a channel from it's ID. Can raise LookupError."""
    for order,channel in self.channels.items() :
      if channel.getId() == channelId :
        return channel
    raise LookupError
  
  def getLicences(self) : # can raise urllib.error (inherits from IOError)
    if len(self.licences) == 0 :
      url = orangetvurl + 'pfs-webapp/user/liveInfos.json'
      urllib.urlretrieve(url, os.path.join(self.cachepath, 'userInfos.json'))
      info = jsonLoad(os.path.join(self.cachepath, 'userInfos.json'))
      if info.has_key('response') :
        self.licences = info['response']['licenceType']
    return self.licences

class Channel :
  
  def __init__(self, chanid, name, arturl, programs, licence="") :
    self.chanid = chanid
    self.name = name
    self.arturl = arturl
    self.programs = programs
    self.licence = licence
  
  def getId(self):
    return self.chanid
  
  def getName(self) :
    return self.name
  
  def getArtUrl(self) :
    return self.arturl
  
  def getAiringProgram(self) :
    for program in self.programs :
      if program.isAiringNow() :
        return program
    raise LookupError('Program not found')
  
  def getNextPrograms(self) :
    nextprograms = list()
    for program in self.programs :
      if program.willAir() :
        nextprograms.append(program)
    return nextprograms
  
  def isAuthorized(self, licences) :
    try:
        result = licences.index(self.licence)
        return True
    except ValueError:
        return False
  
  def getURL(self, highquality, cachepath) :  # can raise urllib.error (inherits from IOError)
    if highquality :
      url = orangetvurl + 'pfs-webapp/user/live/channel/' + str(self.getId()) + '/url.json?resolution=MEDIUM&subtitle=false'
    else :
      url = orangetvurl + 'pfs-webapp/user/live/channel/' + str(self.getId()) + '/url.json?resolution=LOW&subtitle=false'
    urllib.urlretrieve(url, os.path.join(cachepath, 'url.json'))
    info = jsonLoad(os.path.join(cachepath, 'url.json'))
    if info.has_key('response') :
      return re.sub('http://', 'mmsh://CDLYOS02.se.', info['response']['url'])
    else :
      raise IOError

class Program :
  
  def __init__(self, title, genre, date, duration) :
    self.title = title
    self.genre = genre
    self.diffusionDate = date
    self.duration = duration
  
  def __str__(self) :
    return "Program ( Title: %s , Diffusion Date: %s , Duration: %s seconds )" % (self.title, str(self.diffusionDate), str(self.duration))
  
  def __repr__(self):
    return self.__str__()
  
  def isAiringNow(self) :
    delta = datetime.datetime.now()-self.diffusionDate
    deltasecond = delta.days * 24 * 3600 + delta.seconds
    if deltasecond > 0 and deltasecond <= self.duration :
      return True
    else :
      return False
  
  def willAir(self) :
    if self.diffusionDate > datetime.datetime.now() :
      return True
    else :
      return False
  
  def getTitle(self) :
    return self.title
  
  def getGenre(self) :
    return self.genre
  
  def getDate(self) :
    return self.diffusionDate
  
  def getDuration(self, resolution=1) :
    return self.duration/resolution
