#!/usr/bin/python
# -*- coding: utf-8 -*-

###################################################
#
#        WTO est un programme permettant de regarder la WebTv d'Orange (sans Firefox),
#        
#        Copyright 2008 Tuxgasy <tuxgasy@tuxfamily.org>
#        Copyright 2010 Loïc GRENON <difool@tuxfamily.org>
#
#        This program is free software; you can redistribute it and/or modify
#        it under the terms of the GNU General Public License as published by
#        the Free Software Foundation; either version 3 of the License, or
#        (at your option) any later version.
#        
#        This program is distributed in the hope that it will be useful,
#        but WITHOUT ANY WARRANTY; without even the implied warranty of
#        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#        GNU General Public License for more details.
#        
#        You should have received a copy of the GNU General Public License
#        along with this program; if not, write to the Free Software
#        Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#        MA 02110-1301, USA.
#
###################################################

# Gtk
import gtk, gobject, glib

# fonctions système
import os, subprocess, time, tempfile

# requêtes HTTP
import urllib, urllib2

# parseur de config valeur-cle
import ConfigParser

# thread
import thread

# orange, recuperation des chaines
import orange

# Variables globales

WTO_VERSION = '1.0.7'
WTO_DIRNAME = 'wto'

# Emplacements dépendants de l'OS

if 'APPDATA' in os.environ:
  WTO_CONFIG_PATH = os.path.join(os.environ['APPDATA'], WTO_DIRNAME)
elif 'XDG_CONFIG_HOME' in os.environ:
  WTO_CONFIG_PATH = os.path.join(os.environ['XDG_CONFIG_HOME'], WTO_DIRNAME)
else:
  WTO_CONFIG_PATH = os.path.join(os.environ['HOME'], '.config', WTO_DIRNAME)

if 'APPDATA' in os.environ:
  WTO_CACHE_PATH = os.path.join(WTO_CONFIG_PATH, 'cache')
elif 'XDG_CACHE_HOME' in os.environ:
  WTO_CACHE_PATH = os.path.join(os.environ['XDG_CACHE_HOME'], WTO_DIRNAME)
else:
  WTO_CACHE_PATH = os.path.join(os.environ['HOME'], '.cache', WTO_DIRNAME)

# Autre emplacements

WTO_ICON = "/usr/share/icons/hicolor/32x32/apps/wto.png"
if not os.access(WTO_ICON, os.F_OK) :
  WTO_ICON = "icons/wto-32.png"
if not os.access(WTO_ICON, os.F_OK) :
  WTO_ICON = "../icons/wto-32.png"
WTO_LOCK_FILE = os.path.join(WTO_CONFIG_PATH, 'wto.lock')
WTO_CONF = os.path.join(WTO_CONFIG_PATH, 'wto.conf')
WTO_RECORDS_DIR = os.path.join(WTO_CONFIG_PATH, 'records')
WTO_COOKIES_FILE = os.path.join(WTO_CACHE_PATH, 'cookies')
WTO_CHANNELS_ICONS = os.path.join(WTO_CACHE_PATH, 'icons')

# Developpeurs et artistes
WTO_AUTHORS = ['Papaneo', 'TuxGasy (alias UgM) <tuxgasy@tuxfamily.org>', 'Arnaud Durand-Favreau (biginoz) <biginoz@free.fr> ', 'Guillaume Andrieu <andrieuguillaume42@gmail.com>', 'Loïc GRENON <difool@tuxfamily.org>']
WTO_ARTISTS = ['TuxGasy <tuxgasy@tuxfamily.org>']
WTO_LICENSE = """
Wto est un logiciel libre ; vous pouvez le redistribuer et/ou
le modifier selon les termes de la Licence Publique Générale GNU
publiée par la Free Software Foundation ; soit la version 3 de la
license, soit (à votre discrétion) toute version ultérieure.

Wto est distribué dans l'espoir qu'il puisse vous être utile,
mais SANS AUCUNE GARANTIE ; sans même la garantie de VALEUR MARCHANDE
ou d'ADÉQUATION À UN BESOIN PARTICULIER. Consultez la Licence Publique
Générale GNU pour plus de détails.

Vous devez avoir reçu une copie de la Licence Publique Générale GNU
en même temps que Wto ; si ce n'est pas le cas, écrivez à la
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
Boston, MA 02110-1301, États-Unis.
"""

class MainInterface :
  
  config = None
  window, menu, listChannels, statusBar, controlBox, titre = None, None, None, None, None, None
  channels, player = None, None
  
  def quit(self, widget, data=None) :
    if self.player != None :
      self.player.quit()
    if self.config != None : 
      size = self.window.get_size()
      try :
        self.config.config.set('general', 'window_width', size[0])
        self.config.config.set('general', 'window_height', size[1])
      except ConfigParser.NoSectionError :
        pass
      self.config.write()
    gtk.main_quit()
  
  def initListChannels(self) :
    """Initialize the channel list. This must be called within a thread"""
    gtk.gdk.threads_enter()
    self.statusBar.addText("Recuperation de la liste des chaines en cours...")
    gtk.gdk.threads_leave()
    try:
      self.channels = orange.Channels(WTO_CACHE_PATH)
      self.showListChannels()
      gtk.gdk.threads_enter()
      self.statusBar.addText("Recuperation de la liste des chaines... Ok")
      gtk.gdk.threads_leave()
    except urllib2.URLError :
      gtk.gdk.threads_enter()
      self.statusBar.addText("Problème de connexion")
      gtk.gdk.threads_leave()
      print "Problème de connexion"
    
    # Reaffichage de la liste des chaînes, période 1 minute
    gobject.timeout_add(60000, self.onRefreshListChannels, None)
    # Téléchargement de la liste des chaînes, période 10 minute
    gobject.timeout_add(600000, self.onUpdateListChannels, None)
    
  def showListChannels(self) :
    try :
      channels = self.channels.getChannels().items()
    except IOError :
      channels = {}
      gtk.gdk.threads_enter()
      self.showMessageDialog('Impossible d\'obtenir la liste des chaines', 'Erreur réseau', gtk.MESSAGE_ERROR)
      gtk.gdk.threads_leave()
    try :
      hideNotAuthorized = self.config.config.getboolean('general', 'hide_not_authorized')
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
      hideNotAuthorized = True
    gtk.gdk.threads_enter()
    self.listChannels.listStore.clear()
    gtk.gdk.threads_leave()
    self.listChannels.setChannels(channels, self.channels.getLicences(), hideNotAuthorized)
  
  def onRefreshListChannels(self, widget, data=None) :
    thread.start_new_thread(self.showListChannels, ())
    return True
  
  def onUpdateListChannels(self, widget, data=None) :
    thread.start_new_thread(self.updateListChannels, ())
    return True
  
  def onUpdateAllListChannels(self, widget, data=None) :
    thread.start_new_thread(self.updateListChannels, (True,))
  
  def updateListChannels(self, download_icon=False) :
    """Update the channel list, and re-display it. This must be called within a thread."""
    gtk.gdk.threads_enter()
    self.statusBar.addText("Mise à jour de la liste des chaines en cours...")
    gtk.gdk.threads_leave()
    if download_icon :
      listdir = os.listdir(WTO_CHANNELS_ICONS)
      for iconFile in listdir :
        os.remove(os.path.join(WTO_CHANNELS_ICONS, iconFile))
    try :
      self.channels.fetchChannels()
      self.showListChannels()
      gtk.gdk.threads_enter()
      self.statusBar.addText("Mise à jour de la liste des chaines... Ok")
      gtk.gdk.threads_leave()
    except BaseException :
      gtk.gdk.threads_enter()
      self.showMessageDialog('Impossible de mettre a jour les chaines', 'Erreur', gtk.MESSAGE_ERROR)
      gtk.gdk.threads_leave()
    return True
  
  def getChannelId(self, idElement) :
    return self.listChannels.listStore.get(self.listChannels.listStore.get_iter_from_string(str(idElement)), 0)[0]

  def onHideNotAuthorized(self, widget, data=None) :
    try :
      active = self.config.config.getboolean('general', 'hide_not_authorized')
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
      active = True
    if active :
      self.config.config.set('general', 'hide_not_authorized', 'false')
    else :
      self.config.config.set('general', 'hide_not_authorized', 'true')
    self.config.write()
    thread.start_new_thread(self.showListChannels, ())
  
  def onToggleHighQuality(self, widget, data=None) :
    try :
      active = self.config.config.getboolean('general', 'highvideoquality')
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
      active = False
    if active :
      self.config.config.set('general', 'highvideoquality', 'false')
    else :
      self.config.config.set('general', 'highvideoquality', 'true')
    self.config.write()

  def onToggleMplayerOnTop(self, widget, data=None) :
    try :
      active = self.config.config.getboolean('player', 'player_ontop')
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
      active = False
    if active :
      self.config.config.set('player', 'player_ontop', 'false')
    else :
      self.config.config.set('player', 'player_ontop', 'true')
    self.config.write()
  
  def showChannel(self, channelId) :
    """Play a channel stream. This must be called within a thread."""
    try :
      channel = self.channels.getChannelById(self.getChannelId(channelId))
      licences = self.channels.getLicences()
    except :
      gtk.gdk.threads_enter()
      self.showMessageDialog('Doh! Impossible d\'obtenir les informations du serveur d\'Orange', 'Erreur réseau', gtk.MESSAGE_ERROR)
      gtk.gdk.threads_leave()
      return
    print 'Lancement de la chaine #'+str(channel.getId())+' ('+channel.getName()+')...'
    self.statusBar.addText('Lancement de '+channel.getName()+'...')
    
    if channel.isAuthorized(licences) :
      try :
        player_options = self.config.config.get('player', 'options')
      except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
        player_options = ""
      try :
        on_top = self.config.config.getboolean('player', 'player_ontop')
      except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
        on_top = False
      if on_top:
        player_options += ' -ontop'
      try :
        highquality = self.config.config.getboolean('general', 'highvideoquality')
      except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
        highquality = False
      try :
        channel_url = channel.getURL(highquality, WTO_CACHE_PATH)
        self.player.play(channel_url, player_options, channel.getName())
      except IOError :
        gtk.gdk.threads_enter()
        self.showMessageDialog('Impossible d\'obtenir les informations du serveur d\'Orange', 'Erreur réseau', gtk.MESSAGE_ERROR)
        gtk.gdk.threads_leave()
    else :
      message = "Cette chaine ne fait pas partie de votre abonnement.\nPour plus de renseignements, rendez-vous sur la boutique en ligne Orange."
      gtk.gdk.threads_enter()
      self.showMessageDialog(message)
      gtk.gdk.threads_leave()
  
  def recordChannel(self, channelId) :
    channel = self.channels.getChannelById(channelId)
    print 'Enregistrement de la chaine n°'+str(channelId)+' ('+channel.getName()+')...'
    self.statusBar.addText('Enregistrement de '+channel.getName()+'...')
    # Format du nom du fichier record_JJ-MM-AAAA_HH-MM-SS.wmv
    output_file = os.path.join(WTO_RECORDS_DIR, 'record'+'_'+time.strftime("%d-%m-%Y_%H-%M-%S", time.gmtime())+'.wmv')
    try :
      highquality = self.config.config.getboolean('general', 'highvideoquality')
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
      highquality = False
    self.record = Record(channel.getURL(highquality, WTO_CACHE_PATH), output_file)

  def showMessageDialog(self, message, title="Information", diatype=gtk.MESSAGE_WARNING) :
    dialog = gtk.MessageDialog(self.window, gtk.DIALOG_DESTROY_WITH_PARENT, diatype, gtk.BUTTONS_OK, message)
    dialog.set_title(title)
    dialog.run()
    dialog.destroy()

  def showWin(self, status_icon) :
    if(self.window.get_property("visible")) :
      self.window.hide_all()
    else :
      self.window.show_all()
  
  def setplayer(self, menu_item, player) : 
    if(self.player == None or self.player.__class__.__name__ != player) :
      self.player = eval(player)()
      if(self.player.is_available() and menu_item != None) :
        try :
          self.config.config.set('player', 'player', player)
        except ConfigParser.NoSectionError :
          self.config.config.add_section('player')
          self.config.config.set('player', 'player', player)
        self.config.write()
      else :
        self.showMessageDialog(self.player.__class__.__name__+' n\'est pas installé. Installez le ou choisissez un autre lecteur.', "Erreur", gtk.MESSAGE_ERROR)
    
  def __init__(self, config) :
    self.config = config
    print 'Initialisation de la fenêtre principale...'
    self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    self.window.connect("destroy", self.quit)
    self.window.set_title('WTO '+WTO_VERSION)
    
    try :
      width = self.config.config.getint('general', 'window_width')
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
      width = 600
    try :
      height = self.config.config.getint('general', 'window_height')
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
      height = 500
    
    self.window.set_default_size(width, height)
    
    if os.access(WTO_ICON, os.F_OK) :
      try :
        window_icon = gtk.gdk.pixbuf_new_from_file(WTO_ICON)
      except glib.GError :
        window_icon = None
      self.window.set_icon(window_icon)
      status_icon = gtk.status_icon_new_from_file(WTO_ICON)
      status_icon.set_tooltip('Web Tv Orange')
      status_icon.connect("activate", self.showWin)
    
    vbox = gtk.VBox(False, True)
    self.window.add(vbox)

    # Ecoute des evenements pression de bouton
    self.window.add_events(gtk.gdk.BUTTON_PRESS_MASK)
    self.window.connect("key-press-event", self.button_pressed)
    
    # Barre de menu
    self.menu = MenuInterface(self)
    vbox.pack_start(self.menu.menu_bar, False, True)
    
    # Controle de Mplayer
    #self.controlBox = ControlInterface(self)
    #vbox.pack_start(self.controlBox.hbox, False, True, 5)
        
    # box contenant la liste des chaines
    self.listChannels = ListChannelsInterface(self)
    vbox.pack_start(self.listChannels.channelsBox, True, True)
    
    # box contenant des infos sur la chaine
    self.channelInfo = ChannelInfoBox(self)
    vbox.pack_start(self.channelInfo.label, False, False, 0)
    
    # Barre d'état
    self.statusBar = StatusBarInterface(self)
    self.statusBar.addText("Wto "+WTO_VERSION)
    vbox.pack_start(self.statusBar.status_bar, False, False, 0)
    
    self.window.show_all()
    self.channelInfo.hide()

    # Lecteur au demarage
    try :
      player = self.config.config.get('player', 'player')
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
      player = "Mplayer"
    self.setplayer(None, player)

    thread.start_new_thread(self.initListChannels, ())

    gtk.gdk.threads_init()
    gtk.gdk.threads_enter()
    gtk.main()
    gtk.gdk.threads_leave()

  def button_pressed(self, widget, event):
    if(gtk.gdk.keyval_name(event.keyval) == "F5"):
      self.onUpdateListChannels(widget)

class ChannelInfoBox :
  
  def __init__(self, interface) :
    self.interface = interface
    self.label = gtk.Label()
    self.label.set_alignment(0.0, 0.0)
    self.label.set_padding(10, 10)
  
  def hide(self) :
    self.label.hide()
  
  def show(self) :
    self.label.show()
  
  def setText(self, text) :
    self.label.set_markup(text)
  
  def showText(self, text) :
    self.setText(text)
    self.show()
  
  def showInfo(self, elementId) :
    channelId = self.interface.getChannelId(elementId)
    channel = self.interface.channels.getChannelById(channelId)
    text = "<b>A suivre</b>"
    # limit to the 5 next programs
    for program in channel.getNextPrograms()[:5] :
      text += "\n" + str(program.getDate().hour).zfill(2) + "h" + str(program.getDate().minute).zfill(2) + " : " + program.getTitle().replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')
    if len(channel.getNextPrograms()) > 0 :
      self.showText(text)
    else :
      self.hide()

class Config :
  
  config = None
  
  def __init__(self) :
    self.config = ConfigParser.ConfigParser()
    if not os.access(WTO_CONF, os.F_OK | os.W_OK) :
      self.config.add_section('general')
      self.config.set('general', 'highvideoquality', 'false')
      
      self.config.add_section('player')
      self.config.set('player', 'player', 'Mplayer')
      self.config.set('player', 'options', '-cache 400 -cache-min 90')
      self.config.set('player', 'player_ontop', 'true')
      
      self.write()
    else :
      self.config.read(WTO_CONF)

      try :
        self.config.getboolean('general', 'highvideoquality')
      except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
        try :
          self.config.add_section('general')
        except ConfigParser.DuplicateSectionError :
          pass
        self.config.set('general', 'highvideoquality', 'false')
        self.write()

      try :
        self.config.get('player', 'player')
      except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
        self.config.set('player', 'player', 'Mplayer')
        self.write()

      try :
        self.config.getboolean('player', 'player_ontop')
      except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
        self.config.set('player', 'player_ontop', 'true')
        self.write()
  
  def write(self) :
    self.config.write(open(WTO_CONF, 'wb'))
      

class MenuInterface :
  
  menu_bar = None
  
  def __init__(self, data) :
        
    # Barre de menu
    self.menu_bar = gtk.MenuBar()
    
    # Menu Fichier
    menu = gtk.Menu()
    accel_file = gtk.AccelGroup()
    menu_item = gtk.MenuItem(label="_Fichier")
    menu_item.set_submenu(menu)
    self.menu_bar.append(menu_item)
    
    # Les éléments du menu Fichier
    menu_item = gtk.ImageMenuItem(gtk.STOCK_QUIT, accel_file)
    menu_item.set_tooltip_text('Quitter wto')
    menu_item.connect("activate", data.quit)
    menu.append(menu_item)

    data.window.add_accel_group(accel_file)
    
    # Menu Enregistrement
    #menu = gtk.Menu()
    #menu_item = gtk.MenuItem(label="_Enregistrement")
    #menu_item.set_submenu(menu)
    #self.menu_bar.append(menu_item)
    
    # Les éléments du menu Enregistrement
    #menu_item = gtk.MenuItem("Ajouter un enregistrement...")
    #menu_item.set_sensitive(False)
    #menu_item.set_tooltip_text('Enregistrer une chaine')
    #self.menu_item_add_record = menu_item
    #menu_item.connect("activate", self.onAddRecord)
    #menu.append(menu_item)
    
    #menu_item = gtk.MenuItem("Liste des enregistrements...")
    #menu_item.set_tooltip_text('Listes de vos enregistrements')
    #menu_item.connect("activate", self.onShowListRecord)
    #menu.append(menu_item)
    
    # Menu Affichage
    menu = gtk.Menu()
    accel_display = gtk.AccelGroup()
    menu_item = gtk.MenuItem(label="_Affichage")
    menu_item.set_submenu(menu)
    self.menu_bar.append(menu_item)
    
    # Les éléments du menu Affichage
    menu_item = gtk.ImageMenuItem(gtk.STOCK_REFRESH, accel_display)
    menu_item.set_tooltip_text('Actualiser la liste des chaines')
    menu_item.connect("activate", data.onUpdateListChannels)
    menu.append(menu_item)
    
    menu_item = gtk.MenuItem('Actualiser tous')
    menu_item.set_tooltip_text('Actualiser la liste des chaines en retéléchargeant les icônes')
    menu_item.connect("activate", data.onUpdateAllListChannels)
    menu.append(menu_item)
    
    menu_item = gtk.CheckMenuItem('Masquer les chaînes hors abonnement')
    try :
      menu_item.set_active(data.config.config.getboolean('general', 'hide_not_authorized'))
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) : 
      menu_item.set_active(True)
    menu_item.set_tooltip_text('Affichement uniquement les chaînes comprises dans votre abonnement')
    menu_item.connect("toggled", data.onHideNotAuthorized)
    menu.append(menu_item)

    menu_item = gtk.CheckMenuItem('Haute qualité')
    try :
      menu_item.set_active(data.config.config.getboolean('general', 'highvideoquality'))
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
      menu_item.set_active(False)
    menu_item.set_tooltip_text('Visualiser les vidéos en haute qualité')
    menu_item.connect("toggled", data.onToggleHighQuality)
    menu.append(menu_item)

    menu_item = gtk.CheckMenuItem('Premier plan')
    try :
      menu_item.set_active(data.config.config.getboolean('player', 'player_ontop'))
    except (ConfigParser.NoSectionError, ConfigParser.NoOptionError) :
      menu_item.set_active(False)
    menu_item.set_tooltip_text('Mettre la fenêtre de Mplayer au premier plan')
    menu_item.connect("toggled", data.onToggleMplayerOnTop)
    menu.append(menu_item)

    data.window.add_accel_group(accel_display)
    
    # Menu Lecteur
    menu = gtk.Menu()
    menu_item = gtk.MenuItem(label="_Lecteur")
    menu_item.set_submenu(menu)
    self.menu_bar.append(menu_item)
    
    menu_item = gtk.RadioMenuItem(None, "_MPlayer")
    menu_item.connect("activate", data.setplayer, 'Mplayer')
    if data.config.config.get('player', 'player') == 'Mplayer' :
      menu_item.set_active(True)
    menu.append(menu_item)
    
    menu_item = gtk.RadioMenuItem(menu_item, "_Totem")
    menu_item.connect("activate", data.setplayer, 'Totem')
    if data.config.config.get('player', 'player') == 'Totem' :
      menu_item.set_active(True)
    menu.append(menu_item)
    
    menu_item = gtk.RadioMenuItem(menu_item, "_VLC")
    menu_item.connect("activate", data.setplayer, 'VLC')
    if data.config.config.get('player', 'player') == 'VLC' :
      menu_item.set_active(True)
    menu.append(menu_item)
    
    # Menu ?
    menu = gtk.Menu()
    menu_item = gtk.MenuItem(label="?")
    menu_item.set_submenu(menu)
    self.menu_bar.append(menu_item)
    
    # Les éléments du menu ?
    menu_item = gtk.ImageMenuItem(stock_id=gtk.STOCK_ABOUT)
    menu_item.set_tooltip_text('A propos de wto')
    menu_item.connect("activate", About, data)
    menu.append(menu_item)
    
class ControlInterface :
  
  hbox, btn_previous, btn_play, btn_stop, btn_record, btn_next = None, None, None, None, None, None
  
  def OnActiveRecord(self, widget, data=None) :
    if widget.get_active() :
      print 'enregistrer...'
    else :
      print 'stop enregistrement...'
  
  def __init__(self, data) :
    self.hbox = gtk.HBox(True, 0)
    
    self.hbox.hide()
    
    self.btn_previous = gtk.ToolButton(stock_id=gtk.STOCK_MEDIA_PREVIOUS)
    self.btn_previous.set_sensitive(False)
    self.hbox.pack_start(self.btn_previous, True, True, 0)
    
    self.btn_play = gtk.ToolButton(stock_id=gtk.STOCK_MEDIA_PLAY)
    self.btn_play.set_sensitive(False)
    self.hbox.pack_start(self.btn_play, True, True, 0)
    
    self.btn_stop = gtk.ToolButton(stock_id=gtk.STOCK_MEDIA_STOP)
    self.btn_stop.set_sensitive(False)
    self.hbox.pack_start(self.btn_stop, True, True, 0)
    
    self.btn_record = gtk.ToggleToolButton(stock_id=gtk.STOCK_MEDIA_RECORD)
    #self.btn_record.set_sensitive(False)
    self.btn_record.connect("toggled", self.OnActiveRecord)
    self.hbox.pack_start(self.btn_record, True, True, 0)
    
    self.btn_next = gtk.ToolButton(stock_id=gtk.STOCK_MEDIA_NEXT)
    self.btn_next.set_sensitive(False)
    self.hbox.pack_start(self.btn_next, True, True, 0)
    

class About :
  "Fenêtre about"
  
  about = None
  
  def __init__(self, widget, wto) :
    about = gtk.AboutDialog()
    if os.access(WTO_ICON, os.F_OK) :
      try :
        about.set_icon(gtk.gdk.pixbuf_new_from_file(WTO_ICON))
        about.set_logo(gtk.gdk.pixbuf_new_from_file(WTO_ICON))
      except glib.GError :
        pass
    about.set_authors(WTO_AUTHORS)
    about.set_artists(WTO_ARTISTS)
    about.set_license(WTO_LICENSE)
    about.set_name('Wto')
    about.set_version(WTO_VERSION)
    about.set_comments('Wto est un logiciel libre permettant de regarder la\n Web Tv d\'Orange avec Mplayer sans navigateur\n (tel que Firefox)')
    about.set_copyright('Copyright (c) 2009 - Papaneo, TuxGasy (alias UgM)\nCopyright (c) 2010 - Loïc GRENON')
    about.set_website('http://wto.tuxfamily.org')
    
    about.run()
    about.destroy()

class ListChannelsInterface :
  
  channelsBox = None
  listStore = None
  
  def onCursorChanged(self, treeview, data) :
    treeselection = self.treeview.get_selection()
    if treeselection.get_mode() == gtk.SELECTION_SINGLE :
      (model, pathlist) = treeselection.get_selected_rows()
      if len(pathlist) == 0 :
        data.channelInfo.hide()
      else :
        data.channelInfo.showInfo(pathlist[0][0])
  
  def onActiveRow(self, treeview, path, view_column, data) :
    thread.start_new_thread(data.showChannel, (path[0],))

  def on_query_tooltip(self, treeview, x, y, keyboard_tip, tooltip):
    if not treeview.get_tooltip_context(x, y, keyboard_tip):
      return False
    else:
      model, path, it = treeview.get_tooltip_context(x, y, keyboard_tip)
      
      chaine = model.get(it, 2)[0]
      pgm_en_cours = model.get(it, 3)[0]

      tooltip.set_text(chaine+"\n"+pgm_en_cours)
      treeview.set_tooltip_row(tooltip, path)
      return True
  
  def getArtFilename(self, artUrl, download=False) :
    iconFile = os.path.join(WTO_CHANNELS_ICONS, artUrl[artUrl.rfind('/')+1:])
    if download or not os.path.isfile(iconFile) :
      urllib.urlretrieve(artUrl, iconFile)
    return iconFile
  
  def setChannels(self, channels, licences, hideNotAuthorized = True) :
    """Display channels in the channel list. This must be called within a thread."""
    for order, channel in channels :
      if (not hideNotAuthorized) or channel.isAuthorized(licences) :
        gtk.gdk.threads_enter()
        try :
          pixbufIconChannel = gtk.gdk.pixbuf_new_from_file(self.getArtFilename(channel.getArtUrl()))
          pixbufIconChannel = pixbufIconChannel.scale_simple(47, 25, gtk.gdk.INTERP_BILINEAR)
        except glib.GError :
          pixbufIconChannel = None
        try :
          airingprog = channel.getAiringProgram()
          self.listStore.append([channel.getId(), pixbufIconChannel, channel.getName(), airingprog.getTitle(), airingprog.getGenre()])
        except LookupError :
          self.listStore.append([channel.getId(), pixbufIconChannel, channel.getName(), 'Pas de programme', ''])
        gtk.gdk.threads_leave()
  
  def __init__(self, data) :
    scroll_bar = gtk.ScrolledWindow()
    scroll_bar.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
    self.channelsBox = scroll_bar
    
    self.listStore = gtk.ListStore(int, gtk.gdk.Pixbuf, str, str, str)    # Ce ListStore ne sert qu'à pouvoir afficher le Treeview dès le lancement de l'appli
    treeview = gtk.TreeView()
    treeview.set_model(self.listStore)
    self.treeview = treeview
    treeview.props.has_tooltip = True
    treeview.connect("cursor-changed", self.onCursorChanged, data)
    treeview.connect("row-activated", self.onActiveRow, data)
    treeview.connect("query-tooltip", self.on_query_tooltip)
    
    scroll_bar.add(treeview)
    
    cell = gtk.CellRendererText()
    column = gtk.TreeViewColumn('N°', cell, text=0)
    column.set_visible(False)
    treeview.append_column(column)
    
    cell = gtk.CellRendererPixbuf()
    column = gtk.TreeViewColumn('', cell, pixbuf=1)
    treeview.append_column(column)
    
    cell = gtk.CellRendererText()
    column = gtk.TreeViewColumn('Chaines', cell, text=2)
    column.set_resizable(True)
    treeview.append_column(column)
    
    cell = gtk.CellRendererText()
    column = gtk.TreeViewColumn('Programme en cours', cell, text=3)
    column.set_resizable(True)
    treeview.append_column(column)
    
    cell = gtk.CellRendererText()
    column = gtk.TreeViewColumn('Genre', cell, text=4)
    column.set_resizable(True)
    treeview.append_column(column)

class StatusBarInterface :
  
  status_bar, context_id = None, None
  
  def addText(self, msg) :
    self.status_bar.push(self.context_id, msg)
  
  def __init__(self, data) :
    self.status_bar = gtk.Statusbar()
    self.status_bar.show_all()
    self.context_id = self.status_bar.get_context_id("ShowChannel")

# La fonction which() proviens de
# http://stackoverflow.com/questions/377017/test-if-executable-exists-in-python

def which(program):
  
  def is_exe(fpath):
    return os.path.exists(fpath) and os.access(fpath, os.X_OK)
  
  fpath, fname = os.path.split(program)
  
  if fpath:
    if is_exe(program):
      return program
  else:
    for path in os.environ["PATH"].split(os.pathsep):
      exe_file = os.path.join(path, program)
      if is_exe(exe_file):
        return exe_file
  
  return None

class Player(object) :
  
  process = None

  def __init__(self) :
    self.executable = None
  
  def is_available(self) :
    if(which(self.executable) == None) :
      return False
    else :
      return True
  
  def is_running(self) :
    return self.process != None and self.process.poll() == None
  
  def run(self, url, options='', titre='Player') :
    raise NotImplementedError("Player : run not implemented")
  
  def quit(self) :
    if self.is_running() :
      self.process.terminate()
  
  def play(self, url, options='', titre='Player') :
    raise NotImplementedError("Player : play not implemented")
  
  def pause(self) :
    raise NotImplementedError("Player : pause not implemented")
  
  def stop(self) :
    raise NotImplementedError("Player : stop not implemented")

class VLC(Player) : 
  
  def __init__(self) :
    super(VLC, self).__init__()
    self.executable = 'vlc'
  
  def run(self, url, options='', titre='') :
    cmd = self.executable+' --extraintf rc "'+url+'"'
    self.process = subprocess.Popen(cmd, shell=True, bufsize=0, stdin=subprocess.PIPE, stdout=tempfile.NamedTemporaryFile(), stderr=subprocess.STDOUT, close_fds=True)
    
  def play(self, url, options='', titre='') :
    if not self.is_running() :
      self.run(url)
    else :
      self.process.stdin.write('add '+url+'\n')
  
  def pause(self) :
    if self.is_running() :
      self.process.stdin.write('pause\n')
    
  def stop(self) :
    if self.is_running() :
      self.process.stdin.write('stop\n')

  def quit(self) :
    if self.is_running() :
      self.process.stdin.write('quit\n')

class Mplayer(Player) :

  def __init__(self) :
    super(Mplayer, self).__init__()
    self.executable = 'mplayer'
  
  def run(self, url, options='', titre='MPlayer') :
    cmd = self.executable+' -title \''+titre+'\' -slave -idle -quiet '+options+' "'+url+'"'
    self.process = subprocess.Popen(cmd, shell=True, bufsize=0, stdin=subprocess.PIPE, stdout=tempfile.NamedTemporaryFile(), stderr=subprocess.STDOUT, close_fds=True)
    time.sleep(10) # pour laisser le tps à Mplayer de planter !
    if (self.process.poll() == 1) :  # Bug Fix : Mplayer ne supporte pas l'argument -title selon la version
      cmd = 'mplayer -slave -idle -quiet '+options+' "'+url+'"'
      self.process = subprocess.Popen(cmd, shell=True, bufsize=0, stdin=subprocess.PIPE, stdout=tempfile.NamedTemporaryFile(), stderr=subprocess.STDOUT, close_fds=True)
  
  def play(self, url, options='', titre='MPlayer') :
    self.quit()
    self.run(url, options, titre)
  
  def pause(self) :
    if self.is_running() :
      self.process.stdin.write('pause\n')
  
  def stop(self) :
    self.quit()

class Totem(Player) :
  
  def __init__(self) :
    super(Totem, self).__init__()
    self.executable = 'totem'
  
  def run(self, url='', options='', titre='') :
    if(url == '') :
      self.process = subprocess.Popen([self.executable])
    else :
      self.process = subprocess.Popen([self.executable, url])
    
  def play(self, url='', options='', titre='') :
    if self.is_running() :
      subprocess.Popen([self.executable, "--replace", url])
    else :
      self.run(url)
  
  def pause(self) :
    if self.is_running() :
      subprocess.Popen([self.executable, "--pause"])
    
  def stop(self) :
    if self.is_running() :
      self.quit()

class Record :
  record = None
  
  def start(self, url, output) :
    cmd = 'mimms '+url+' '+output
    self.record = subprocess.Popen(cmd, shell=True, bufsize=0, stdin=subprocess.PIPE, stdout=tempfile.NamedTemporaryFile(), stderr=subprocess.STDOUT, close_fds=True)
  
  def stop(self) :
    self.record.terminate()
  
  def __init__(self, url, output) :
    self.start(url, output)

def main() :
  print 'Wto (version '+WTO_VERSION+') - La WebTv d\'Orange sans Firefox'
  print """
Ce programme est sous licence GPL v3
  
(c) 2009 - TuxGasy <tuxgasy@tuxfamily.org>
(c) 2010 - Loïc GRENON <difool@tuxfamily.org>
  """
  
  #if os.access(WTO_LOCK_FILE, os.F_OK) :
    #return
  
  #création du fichier de lock
  #os.mknod(WTO_LOCK_FILE)
  
  # création du dossier de configuration
  if not os.access(WTO_CONFIG_PATH, os.F_OK | os.W_OK) :
    print 'Création du répertoire '+WTO_CONFIG_PATH+'...'
    os.mkdir(WTO_CONFIG_PATH)
  
  # création du dossier de cache
  if not os.access(WTO_CACHE_PATH, os.F_OK | os.W_OK) :
    print 'Création du répertoire '+WTO_CACHE_PATH+'...'
    os.mkdir(WTO_CACHE_PATH)
  
  # création du dossier d'enregistrement
  if not os.access(WTO_RECORDS_DIR, os.F_OK | os.W_OK) :
    print 'Création du répertoire '+WTO_RECORDS_DIR+'...'
    os.mkdir(WTO_RECORDS_DIR)
  
  # création du dossier de cache des icônes
  if not os.access(WTO_CHANNELS_ICONS, os.F_OK | os.W_OK) :
    print 'Création du répertoire '+WTO_CHANNELS_ICONS+'...'
    os.mkdir(WTO_CHANNELS_ICONS)

  config = Config()
  MainInterface(config)
  
  # Effacer le fichier de lock
  #os.remove(WTO_LOCK_FILE)
